#include-once

Global Const $EXIT_FAILURE = 1
Global Const $EXIT_SUCCESS = 0

Global $__RMonitor__FolderIsSet = False
Global $__RMonitor__CallbackIsSet = False

Global $FILE_ACTION_ADDED = 1
Global $FILE_ACTION_REMOVED = 2
Global $FILE_ACTION_MODIFIED = 3
Global $FILE_ACTION_RENAMED_OLD_NAME = 4
Global $FILE_ACTION_RENAMED_NEW_NAME = 5

Global $__RMonitorDLL = DllOpen("RMonitor.dll")
If @error Then
	ConsoleWriteError("Failed to load RMonitor.dll")
	Exit 1
EndIf

OnAutoItExitRegister("__RMonitor__CloseDLL")


; #FUNCTION# ====================================================================================================================
; Name ..........: RMonitor_SetFolder
; Description ...: Set a folder to monitor
; Syntax ........: RMonitor_SetFolder($Folder)
; Parameters ....: $Folder              - Path to the folder (UNICODE is supported)
; Return values .: $EXIT_SUCCESS if success, non-zero value if error occurs
; Author ........: Jonathan S. Hopper (Mano78170)
; ===============================================================================================================================
Func RMonitor_SetFolder($Folder)
	Local $_Rep

	$_Rep = DllCall($__RMonitorDLL, "int:cdecl", "SetMonitoringFolder", "wstr", $Folder)
	If @error Then
		Return $EXIT_FAILURE
	ElseIf $_Rep[0] == $EXIT_FAILURE Then
		Return $EXIT_FAILURE + 1
	EndIf

	$__RMonitor__FolderIsSet = True
	Return $EXIT_SUCCESS
EndFunc   ;==>RMonitor_SetFolder


; #FUNCTION# ====================================================================================================================
; Name ..........: RMonitor_SetCallback
; Description ...: Set function that RMonitor.dll will callback at event
; Syntax ........: RMonitor_SetCallback($CallbackName)
; Parameters ....: $CallbackName        - Name of a function
; Return values .: $EXIT_SUCCESS if success, non-zero value if error occurs
; Author ........: Jonathan S. Hopper (Mano78170)
; Remarks .......: The fuction MUST have ONLY ONE parameter as WSTR for receive event information
; ===============================================================================================================================
Func RMonitor_SetCallback($CallbackName)
	Local $Callback, $_Rep

	$Callback = DllCallbackRegister($CallbackName, "none", "wstr")
	If $Callback == 0 Then
		Return $EXIT_FAILURE
	EndIf

	$_Rep = DllCall($__RMonitorDLL, "none:cdecl", "SetMonitoringCallback", "ptr", DllCallbackGetPtr($Callback))
	If @error Then
		Return $EXIT_FAILURE + 1
	EndIf

	$__RMonitor__CallbackIsSet = True
	Return $EXIT_SUCCESS
EndFunc   ;==>RMonitor_SetCallback


; #FUNCTION# ====================================================================================================================
; Name ..........: RMonitor_WaitForEvents
; Description ...: Wait for any file or folder event in monitored folder
; Syntax ........: RMonitor_WaitForEvents()
; Return values .: $EXIT_SUCCESS if success, non-zero value if error occurs
; Author ........: Jonathan S. Hopper (Mano78170)
; Remarks .......: RMonitor_SetCallback() AND RMonitor_SetFolder() MUST have been called before this function
;                  (RMonitor_StartMonitoring() is NOT needed for)
;
;				   The callback function can be called several times in one execution of this function
; ===============================================================================================================================
Func RMonitor_WaitForEvents()
	Local $_Rep

	If $__RMonitor__FolderIsSet And $__RMonitor__CallbackIsSet Then

		$_Rep = DllCall($__RMonitorDLL, "int:cdecl", "GetMonitoringResult")
		If @error Then
			Return $EXIT_FAILURE
		ElseIf $_Rep[0] == $EXIT_FAILURE Then
			Return $EXIT_FAILURE + 1
		EndIf
	Else
		Return $EXIT_FAILURE + 2
	EndIf
EndFunc   ;==>RMonitor_WaitForEvents


; #FUNCTION# ====================================================================================================================
; Name ..........: RMonitor_StartMonitoring
; Description ...: Start monitoring dll thread
; Syntax ........: RMonitor_StartMonitoring()
; Return values .: $EXIT_SUCCESS if success, $EXIT_FAILURE if error occurs
; Author ........: Jonathan S. Hopper (Mano78170)
; Remarks .......: RMonitor_SetCallback() AND RMonitor_SetFolder() MUST have been called before this function
; ===============================================================================================================================
Func RMonitor_StartMonitoring()
	Local $_Rep

	$_Rep = DllCall($__RMonitorDLL, "int:cdecl", "StartThreadMonitoring")
	If @error Then
		Return $EXIT_FAILURE
	EndIf

	Return $EXIT_SUCCESS
EndFunc   ;==>RMonitor_StartMonitoring


; #FUNCTION# ====================================================================================================================
; Name ..........: RMonitor_StopMonitoring
; Description ...: Stop monitoring dll thread
; Syntax ........: RMonitor_StopMonitoring()
; Return values .: $EXIT_SUCCESS if success, $EXIT_FAILURE if error occurs
; Author ........: Jonathan S. Hopper (Mano78170)
; Remarks .......: RMonitor_StartMonitoring() MUST have been called before this function
; ===============================================================================================================================
Func RMonitor_StopMonitoring()
	Local $_Rep

	$_Rep = DllCall($__RMonitorDLL, "int:cdecl", "StopThreadMonitoring")
	If @error Then
		Return $EXIT_FAILURE
	EndIf

	Return $EXIT_SUCCESS
EndFunc   ;==>RMonitor_StopMonitoring



; ===============================================================================================================================
; ===============================================================================================================================
; ####################################################   INTERNAL USE ONLY   ####################################################
; ===============================================================================================================================
; ===============================================================================================================================

Func __RMonitor__CloseDLL()
	DllClose($__RMonitorDLL)
EndFunc   ;==>__RMonitor__CloseDLL